/* ===========================================================
# sphere-orderstatus-xml-import - v0.1.0
# ==============================================================
# Copyright (c) 2013,2014 Sven Mueller
# Licensed under the MIT license.
*/
var CHANNEL_KEY, CHANNEL_ROLE, LOG_PREFIX, OrderStatusImport, OrderSync, Promise, SphereClient, package_json, xmlHelpers, _, _ref,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };

_ = require('underscore');

_.mixin(require('underscore-mixins'));

Promise = require('bluebird');

_ref = require('sphere-node-sdk'), SphereClient = _ref.SphereClient, OrderSync = _ref.OrderSync;

package_json = require('../package.json');

xmlHelpers = require('./xmlhelpers');

LOG_PREFIX = "[SphereOrderStatusImport] ";

CHANNEL_KEY = 'OrderStatusImport';

CHANNEL_ROLE = 'OrderImport';

OrderStatusImport = (function() {
  function OrderStatusImport(logger, options) {
    this.logger = logger;
    if (options == null) {
      options = {};
    }
    this._mapXML = __bind(this._mapXML, this);
    this.performXML = __bind(this.performXML, this);
    this.sync = new OrderSync;
    this.client = new SphereClient(options);
  }

  OrderStatusImport.prototype.run = function(fileContent) {
    return this.client.channels.ensure(CHANNEL_KEY, CHANNEL_ROLE).then((function(_this) {
      return function(result) {
        _this.channel = result.body;
        return _this.performXML(fileContent);
      };
    })(this));
  };

  OrderStatusImport.prototype.performXML = function(fileContent) {
    return new Promise((function(_this) {
      return function(resolve, reject) {
        return xmlHelpers.xmlTransform(xmlHelpers.xmlFix(fileContent), function(err, xml) {
          if (err != null) {
            return reject("" + LOG_PREFIX + "Error on parsing XML: " + err);
          } else {
            return _this._mapXML(xml.order).then(function(result) {
              return resolve(result);
            })["catch"](function(err) {
              return reject(err);
            }).done();
          }
        });
      };
    })(this));
  };

  OrderStatusImport.prototype._parcelExists = function(order, trackingId) {
    var result;
    return result = _.chain(order.shippingInfo.deliveries).map(function(delivery) {
      return delivery.parcels;
    }).flatten().find(function(parcel) {
      return parcel.trackingData.trackingId === trackingId;
    }).some().value();
  };

  OrderStatusImport.prototype._mapXML = function(orderStatus) {
    return this.client.orders.where("orderNumber=\"" + orderStatus.orderNumber + "\"").fetch().then((function(_this) {
      return function(result) {
        var changedOrder, originalOrder, syncedActions;
        originalOrder = result.body.results[0];
        if (result.body.total === 0) {
          return Promise.reject("" + LOG_PREFIX + " No order found with orderNumber '" + orderStatus.orderNumber + "'.");
        } else if (result.body.total === 1) {
          changedOrder = _this._mergeOrder(originalOrder, orderStatus);
          syncedActions = _this.sync.buildActions(changedOrder, originalOrder);
          if (syncedActions.shouldUpdate()) {
            return _this.client.orders.byId(syncedActions.getUpdateId()).update(syncedActions.getUpdatePayload()).then(function(result) {
              var updatedOrder;
              updatedOrder = result.body;
              if (result.statusCode !== 200) {
                return Promise.reject("" + LOG_PREFIX + " Error while updating '" + orderStatus.orderNumber + "'.");
              } else {
                return _this._syncOrder(updatedOrder, orderStatus.shippingInfo.deliveries.parcels.trackingData.trackingId).then(function() {
                  return result;
                });
              }
            });
          } else {
            return Promise.resolve({
              statusCode: 304
            });
          }
        }
      };
    })(this));
  };

  OrderStatusImport.prototype._mergeOrder = function(originalOrder, orderStatus) {
    var changedOrder;
    changedOrder = _.deepClone(originalOrder);
    orderStatus.shippingInfo.deliveries.parcels.trackingData.isReturn = orderStatus.shippingInfo.deliveries.parcels.trackingData.isReturn === 'true' ? true : false;
    changedOrder.orderState = orderStatus.orderState;
    changedOrder.shipmentState = orderStatus.shipmentState;
    if (!this._parcelExists(originalOrder, orderStatus.shippingInfo.deliveries.parcels.trackingData.trackingId)) {
      changedOrder.shippingInfo.deliveries.push({
        'parcels': [orderStatus.shippingInfo.deliveries.parcels],
        'items': _.map(originalOrder.lineItems, function(item) {
          return {
            id: item.id,
            quantity: item.quantity
          };
        })
      });
    }
    return changedOrder;
  };

  OrderStatusImport.prototype._syncOrder = function(order, externalId) {
    var data;
    data = {
      version: order.version,
      actions: [
        {
          action: 'updateSyncInfo',
          channel: {
            typeId: 'channel',
            id: this.channel.id
          },
          externalId: externalId
        }
      ]
    };
    return this.client.orders.byId(order.id).update(data);
  };

  return OrderStatusImport;

})();

module.exports = OrderStatusImport;
